###############################################################################
#
#  OptimizationHelper.py --
#
#  $LastChangedDate$
#  $LastChangedRevision$
#  $LastChangedBy$
#
#  Author: jmblanch
#
#  History: 
#    March 5, 2013 - initial revision
#
###############################################################################
import clr
import os
import string

clr.AddReference('Ans.DesignXplorer.PublicAPIs')

from Ans.ComponentSystem import (
    IListCpp, VariantColl
)

from Ansys.DesignXplorer.API.Optimization import (
    IOptimizationServices, 
    IOptimizationPoint
)

class ServicesHelper(object):
    def __init__(self,value):
         self._services = value
		 
    def PublishLogMessage(self, message):
         method = clr.GetClrType(IOptimizationServices).GetMethod("PublishLogMessage")
         arr = System.Array[System.Object]([message])
         method.Invoke(self._services, arr)
		 
    def PublishProgress(self, message, value):
         method = clr.GetClrType(IOptimizationServices).GetMethod("PublishProgress")
         arr = System.Array[System.Object]([message, value])
         method.Invoke(self._services, arr)
		 
    def CalculatePoint(self, PointToCalculate):
         method = clr.GetClrType(IOptimizationServices).GetMethod("CalculatePoint")
         arr = System.Array[System.Object]([PointToCalculate.Point])
         method.Invoke(self._services, arr)
		 
    def CalculatePoints(self, PointsToCalculate):
         method = clr.GetClrType(IOptimizationServices).GetMethod("CalculatePoints")
         Points = VariantColl()
         ind = 0
         for i in range(IListCpp.Count.GetValue(PointsToCalculate)):
             IListCpp.Add(Points,IListCpp.__getitem__(PointsToCalculate,ind).Point)
             ind = ind + 1
         arr = System.Array[System.Object]([Points])
         method.Invoke(self._services, arr)
    
    def CalculateParameterRelationshipsFromPoint(self, Point):
         method = clr.GetClrType(IOptimizationServices).GetMethod("CalculateParameterRelationshipsFromPoint")
         arr = System.Array[System.Object]([Point.Point])
         method.Invoke(self._services, arr)
		 
    def CalculateParameterRelationshipsFromPoints(self, Points):
         method = clr.GetClrType(IOptimizationServices).GetMethod("CalculateParameterRelationshipsFromPoints")
         PointsList = VariantColl()
         ind = 0
         for i in range(IListCpp.Count.GetValue(Points)):
             IListCpp.Add(PointsList,IListCpp.__getitem__(Points,ind).Point)
             ind = ind + 1
         arr = System.Array[System.Object]([PointsList])
         method.Invoke(self._services, arr)
		 
    def PushHistoryPoint(self, pt):
         method = clr.GetClrType(IOptimizationServices).GetMethod("PushHistoryPoint")
         arr = System.Array[System.Object]([pt.Point])
         method.Invoke(self._services, arr)
		 
    def PushConvergenceData(self, data):
         method = clr.GetClrType(IOptimizationServices).GetMethod("PushConvergenceData")
         arr = System.Array[System.Object]([data])
         method.Invoke(self._services, arr)
		 		 
    def _GetExistingPoints(self):
         method = clr.GetClrType(IOptimizationServices).GetProperty("ExistingPoints")
         Points =  method.GetValue(self._services, None)
         PointsHelper = VariantColl()
         if Points != None : 
             listcpp = ListCppHelper(Points)
             for i in range(listcpp.Count):
                 IListCpp.Add(PointsHelper,PointHelper(listcpp[i]))
         return PointsHelper
		 
    ExistingPoints = property(_GetExistingPoints)
	
    def _GetStopped(self):
         method = clr.GetClrType(IOptimizationServices).GetProperty("Stopped")
         return method.GetValue(self._services, None)

    Stopped = property(_GetStopped)
# end of the class

class PointHelper(object):
    def __init__(self,pt):
         self._point = pt
		 
    def _GetPoint(self):
         return self._point

    Point = property(_GetPoint)
	
	
    def get_Value(self,id):
         method = clr.GetClrType(IOptimizationPoint).GetMethod("get_Value")
         arr = System.Array[System.Object]([id])
         return method.Invoke(self._point, arr)
		 
    def put_Value(self,id,value):
         method = clr.GetClrType(IOptimizationPoint).GetMethod("put_Value")
         arr = System.Array[System.Object]([id,value])
         method.Invoke(self._point, arr)	 
		 
    def _SetFeasible(self, value):
         method = clr.GetClrType(IOptimizationPoint).GetProperty("Feasible")
         method.SetValue(self._point, value, None)
		
    def _GetFeasible(self):
         method = clr.GetClrType(IOptimizationPoint).GetProperty("Feasible")
         return method.GetValue(self._point, None)
		 
    Feasible = property(_GetFeasible, _SetFeasible)
	
	
    def _SetParetoFrontIndex(self,value):
         method = clr.GetClrType(IOptimizationPoint).GetProperty("ParetoFrontIndex")
         method.SetValue(self._point, value, None)

    def _GetParetoFrontIndex(self):
         method = clr.GetClrType(IOptimizationPoint).GetProperty("ParetoFrontIndex")
         return method.GetValue(self._point, None)
		 
    ParetoFrontIndex = property(_GetParetoFrontIndex, _SetParetoFrontIndex)
	
	
    def _SetID(self,value):
         method = clr.GetClrType(IOptimizationPoint).GetProperty("ID")
         method.SetValue(self._point, value, None)
		 
    def _GetID(self):
         method = clr.GetClrType(IOptimizationPoint).GetProperty("ID")
         return method.GetValue(self._point, None)

    ID = property(_GetID, _SetID)
	 
	
    def _SetState(self,Value):
         method = clr.GetClrType(IOptimizationPoint).GetProperty("State")
         method.SetValue(self._point, value, None)

    def _GetState(self):
         method = clr.GetClrType(IOptimizationPoint).GetProperty("State")
         return method.GetValue(self._point, None)
		 
    State = property(_GetState, _SetState)
	
	
    def _SetStateMessage(self, value):
         method = clr.GetClrType(IOptimizationPoint).GetProperty("StateMessage")
         method.SetValue(self._point, value, None)
		 
    def _GetStateMessage(self):
         method = clr.GetClrType(IOptimizationPoint).GetProperty("StateMessage")
         return method.GetValue(self._point, None)
		 
    StateMessage = property(_GetStateMessage, _SetStateMessage)    
		
# end of the class

class ListCppHelper(object):
	def __init__(self, listcpp):
		self._listcpp = listcpp

	def _GetCount(self):
		method = clr.GetClrType(IListCpp).GetProperty("Count")
		return method.GetValue(self._listcpp, None)

	Count = property(_GetCount)

	def __getitem__(self,id):
		method = clr.GetClrType(IListCpp).GetProperty("Item")
		arr = System.Array[System.Object]([id])
		return method.GetValue(self._listcpp, arr)

# end of the class
